@@ -1,8 +1,9 @@
 """Ontolutils package."""
 import logging
 
+from ._cfg import set_config, get_config
 from ._version import __version__
-from .classes import Thing, get_urirefs, get_namespaces
+from .classes import Thing, get_urirefs, get_namespaces, as_id, build, Property
 from .classes import namespaces, urirefs
 from .classes import query, dquery
 from .classes.utils import merge_jsonld
@@ -39,5 +40,8 @@ __all__ = ['Thing',
            'dquery',
            'get_urirefs',
            'get_namespaces',
-           'parse_unit'
+           'parse_unit',
+           'as_id',
+           'build',
+           'Property'
            ]
