@@ -2,7 +2,7 @@
 import json
 import logging
 import pathlib
-from typing import Union, Dict, List, Optional
+from typing import Union, Dict, List, Optional, Type
 
 import rdflib
 
@@ -60,7 +60,19 @@ WHERE {
                     if isinstance(o, rdflib.BNode):
                         logger.debug(f'"{o}" is a blank node. Need to find children of it...')
                         _, key = split_URIRef(p)
-                        sub_data[key] = process_object(_id, p, o, graph, add_type)
+                        if key in sub_data:
+                            if isinstance(sub_data[key], list):
+                                sub_data[key].append(process_object(_id, p, o, graph, add_type))
+                            else:
+                                sub_data[key] = [sub_data[key], process_object(_id, p, o, graph, add_type)]
+                        else:
+                            sub_data[key] = process_object(_id, p, o, graph, add_type)
+                    elif str(o).startswith('http'):
+                        # it might be a IRI which is defined inside the JSON-LD:
+                        _sub_data = process_object(_id, p, o, graph, add_type)
+                        if _sub_data:
+                            _, key = split_URIRef(p)
+                            sub_data[key] = _sub_data
                     else:
                         logger.debug(f'dont know what to do with {p} and {o}')
         if predicate in sub_data:
@@ -117,7 +129,13 @@ def _query_by_id(graph, _id: Union[str, rdflib.URIRef], add_type: bool) -> Dict:
             # in this case, we return the object as a string
             out[key] = str(obj)
         else:
-            out[key] = process_object(_id, predicate, obj, graph, add_type)
+            if key in out:
+                if isinstance(out[key], list):
+                    out[key].append(process_object(_id, predicate, obj, graph, add_type))
+                else:
+                    out[key] = [out[key], process_object(_id, predicate, obj, graph, add_type)]
+            else:
+                out[key] = process_object(_id, predicate, obj, graph, add_type)
 
     return out
 
@@ -218,7 +236,7 @@ def dquery(subject: str,
     return [v for v in kwargs.values()]
 
 
-def query(cls: Thing,
+def query(cls: Type[Thing],
           source: Optional[Union[str, pathlib.Path]] = None,
           data: Optional[Union[str, Dict]] = None,
           context: Optional[Union[Dict, str]] = None,
