@@ -3,26 +3,49 @@ import json
 import logging
 import pathlib
 import warnings
+from dataclasses import dataclass
 from datetime import datetime
-from typing import Dict, Union, Optional
+from typing import Dict, Union, Optional, Any, List, Type
 
 import rdflib
-from pydantic import HttpUrl, FileUrl, BaseModel, ConfigDict, field_validator, Field
+from pydantic import HttpUrl, FileUrl, BaseModel, ConfigDict, Field
+from pydantic import field_validator
 from rdflib.plugins.shared.jsonld.context import Context
 
 from .decorator import urirefs, namespaces, URIRefManager, NamespaceManager, _is_http_url
 from .typing import BlankNodeType
 from .utils import split_URIRef
+from .. import get_config
+from ..config import PYDANTIC_EXTRA
 
 logger = logging.getLogger('ontolutils')
 
-EXTRA = 'allow'  # or 'forbid' or 'ignore'
+
+@dataclass
+class Property:
+    name: str
+    property_type: Any
+    default: Optional[Any] = None
+    namespace: Optional[HttpUrl] = None
+    namespace_prefix: Optional[str] = None
+
+    def __post_init__(self):
+        if self.namespace is None and self.namespace_prefix is not None:
+            raise ValueError("If namespace_prefix is given, then namespace must be given as well.")
+        if self.namespace_prefix is None and self.namespace is not None:
+            raise ValueError("If namespace is given, then namespace_prefix must be given as well.")
 
 
 class ThingModel(BaseModel, abc.ABC, validate_assignment=True):
     """Abstract base model class to be used by model classes used within ontolutils"""
 
-    model_config = ConfigDict(extra=EXTRA, populate_by_name=True)
+    model_config = ConfigDict(extra=PYDANTIC_EXTRA, populate_by_name=True)
+
+    def __getattr__(self, item):
+        for field, meta in self.model_fields.items():
+            if meta.alias == item:
+                return getattr(self, field)
+        return super().__getattr__(item)
 
     @abc.abstractmethod
     def _repr_html_(self) -> str:
@@ -44,8 +67,15 @@ def resolve_iri(key_or_iri: str, context: Context) -> Optional[str]:
             return 'http://www.w3.org/2000/01/rdf-schema#label'
     return
 
-def build_blank_n3():
-    return rdflib.BNode().n3()
+
+def build_blank_n3() -> str:
+    _blank_node = rdflib.BNode().n3()
+    bnode_prefix_name = get_config("blank_node_prefix_name")
+    if bnode_prefix_name is None:
+        return _blank_node
+    return _blank_node.replace('_:', bnode_prefix_name)
+
+
 @namespaces(owl='http://www.w3.org/2002/07/owl#',
             rdfs='http://www.w3.org/2000/01/rdf-schema#')
 @urirefs(Thing='owl:Thing', label='rdfs:label')
@@ -56,7 +86,7 @@ class Thing(ThingModel):
 
     Example for `prov:Person`:
 
-    >>> @namespaces(prov='http://www.w3.org/ns/prov#',
+    >>> @namespaces(prov='https://www.w3.org/ns/prov#',
     >>>             foaf='http://xmlns.com/foaf/0.1/')
     >>> @urirefs(Person='prov:Person', first_name='foaf:firstName')
     >>> class Person(Thing):
@@ -70,7 +100,7 @@ class Thing(ThingModel):
     >>> print(p.model_dump_jsonld())
     >>> {
     >>>     "@context": {
-    >>>         "prov": "http://www.w3.org/ns/prov#",
+    >>>         "prov": "https://www.w3.org/ns/prov#",
     >>>         "foaf": "http://xmlns.com/foaf/0.1/",
     >>>         "first_name": "foaf:firstName"
     >>>     },
@@ -102,9 +132,23 @@ class Thing(ThingModel):
     @classmethod
     def _id(cls, id: Optional[Union[str, HttpUrl, FileUrl, BlankNodeType]]) -> str:
         if id is None:
-            return rdflib.BNode().n3()
+            return build_blank_n3()
         return str(id)
 
+    @classmethod
+    def build(cls, namespace: HttpUrl,
+              namespace_prefix: str,
+              class_name: str,
+              properties: List[Union[Property, Dict]]) -> type:
+        """Build a Thing object"""
+        return build(
+            namespace,
+            namespace_prefix,
+            class_name,
+            properties,
+            cls
+        )
+
     def __lt__(self, other: ThingModel) -> bool:
         """Less than comparison. Useful to sort Thing objects.
         Comparison can only be done with other Thing objects and if an ID is given.
@@ -283,14 +327,67 @@ class Thing(ThingModel):
             "@context": at_context,
             **serialization
         }
+
+        properties = self.__class__.model_json_schema().get("properties", {})
+        if not properties:
+            properties = self.__class__.model_json_schema().get("items", {}).get(self.__class__.__name__, {}).get(
+                "properties", {})
+
+        for field_name, field_value in properties.items():
+            _use_as_id = field_value.get("use_as_id", None)
+            if _use_as_id is not None:
+                warnings.warn("The use_as_id field is deprecated. Use the @id field instead.", DeprecationWarning)
+                _id = getattr(self, field_name)
+                if _id is not None:
+                    if str(_id).startswith(("_:", "http")):
+                        jsonld["@id"] = getattr(self, field_name)
+                    else:
+                        raise ValueError(f'The ID must be a valid IRI or blank node but got "{_id}".')
         return jsonld
 
+    def serialize(self,
+                  format: str,
+                  context: Optional[Dict] = None,
+                  exclude_none: bool = True,
+                  resolve_keys: bool = True,
+                  assign_bnode: bool = True,
+                  **kwargs) -> str:
+        """
+        Serialize the object to a given format. This method calls rdflib.Graph().parse(),
+        so the available formats are the same as for the rdflib library:
+            ``"xml"``, ``"n3"``,
+           ``"turtle"``, ``"nt"``, ``"pretty-xml"``, ``"trix"``, ``"trig"``,
+           ``"nquads"``, ``"json-ld"`` and ``"hext"`` are built in.
+        The kwargs are passed to rdflib.Graph().parse()
+        """
+
+        jsonld_dict = self.get_jsonld_dict(
+            context=context,
+            exclude_none=exclude_none,
+            resolve_keys=resolve_keys,
+            assign_bnode=assign_bnode
+        )
+        jsonld_str = json.dumps(jsonld_dict)
+
+        logger.debug(f'Parsing the following jsonld dict to the RDF graph: {jsonld_str}')
+        g = rdflib.Graph()
+        g.parse(data=jsonld_str, format='json-ld')
+
+        _context = jsonld_dict.get('@context', {})
+        if context:
+            _context.update(context)
+
+        return g.serialize(format=format,
+                           context=_context,
+                           **kwargs)
+
     def model_dump_jsonld(self,
                           context: Optional[Dict] = None,
                           exclude_none: bool = True,
                           rdflib_serialize: bool = False,
                           resolve_keys: bool = True,
-                          assign_bnode: bool = True) -> str:
+                          assign_bnode: bool = True,
+                          indent: int = 4) -> str:
         """Similar to model_dump_json() but will return a JSON string with
         context resulting in a JSON-LD serialization. Using `rdflib_serialize=True`
         will use the rdflib to serialize. This will make the output a bit cleaner
@@ -315,6 +412,8 @@ class Thing(ThingModel):
             explained in the context.
         assign_bnode: bool=True
             Assigns a blank node if no ID is set.
+        indent: int=4
+            The indent of the JSON-LD string
 
             .. seealso:: `Thing.get_jsonld_dict`
 
@@ -343,7 +442,7 @@ class Thing(ThingModel):
 
         return g.serialize(format='json-ld',
                            context=_context,
-                           indent=4)
+                           indent=indent)
 
     def __repr__(self, limit: Optional[int] = None):
         _fields = {k: getattr(self, k) for k in self.model_fields if getattr(self, k) is not None}
@@ -367,7 +466,7 @@ class Thing(ThingModel):
     @classmethod
     def from_jsonld(cls,
                     source: Optional[Union[str, pathlib.Path]] = None,
-                    data: Optional[Dict] = None,
+                    data: Optional[Union[str, Dict]] = None,
                     limit: Optional[int] = None,
                     context: Optional[Dict] = None):
         """Initialize the class from a JSON-LD source
@@ -380,8 +479,8 @@ class Thing(ThingModel):
         ----------
         source: Optional[Union[str, pathlib.Path]]=None
             The source of the JSON-LD data (filename). Must be given if data is None.
-        data: Optional[str]=None
-            The JSON-LD data as a string. Must be given if source is None.
+        data: Optional[Union[str, Dict]]=None
+            The JSON-LD data as a str or dictionary. Must be given if source is None.
         limit: Optional[int]=None
             The limit of the number of objects to return. If None, all objects will be returned.
             If limit is 1, then the first object will be returned, else a list of objects.
@@ -391,7 +490,9 @@ class Thing(ThingModel):
 
         """
         from . import query
-        if data is not None and isinstance(data, str):
+        if data is not None:
+            if isinstance(data, dict):
+                data = json.dumps(data)
             if 'http://schema.org/' in data:
                 warnings.warn('Replacing http with https in the JSON-LD data. '
                               'This is a workaround for the schema.org inconsistency.',
@@ -451,3 +552,66 @@ def get_urirefs(cls: Thing) -> Dict:
 def get_namespaces(cls: Thing) -> Dict:
     """Return the namespaces of the class"""
     return NamespaceManager[cls]
+
+
+def build(
+        namespace: HttpUrl,
+        namespace_prefix: str,
+        class_name: str,
+        properties: List[Union[Property, Dict]],
+        baseclass=Thing) -> Type[Thing]:
+    """Build a ThingModel class
+
+    Parameters
+    ----------
+    namespace: str
+        The namespace of the class
+    namespace_prefix: str
+        The namespace prefix of the class
+    class_name: str
+        The name of the class
+    properties: Dict[str, Union[str, int, float, bool, datetime, BlankNodeType, None]]
+        The properties of the class
+    baseclass: Type[Thing]
+        The base class to inherit from, default is Thing
+
+
+    Returns
+    -------
+    Thing
+        A Thing
+    """
+    _properties = []
+    for prop in properties:
+        if isinstance(prop, dict):
+            _properties.append(Property(**prop))
+        else:
+            _properties.append(prop)
+
+    annotations = {prop.name: prop.property_type for prop in _properties}
+    default_values = {prop.name: prop.default for prop in _properties}
+
+    new_cls = type(
+        class_name,
+        (baseclass,),
+        {
+            "__annotations__": annotations,  # Define field type
+            **default_values,
+        }
+    )
+    from ontolutils.classes.decorator import _decorate_urirefs, _add_namesapces
+    _urirefs = {class_name: f"{namespace_prefix}:{class_name}"}
+    _namespaces = {namespace_prefix: namespace}
+    for prop in _properties:
+        _ns = prop.namespace
+        _nsp = prop.namespace_prefix
+        if _ns is None:
+            _ns = namespace
+            _nsp = namespace_prefix
+        _urirefs[prop.name] = f"{_nsp}:{prop.name}"
+        if _nsp not in _namespaces:
+            _namespaces[_nsp] = _ns
+
+    _decorate_urirefs(new_cls, **_urirefs)
+    _add_namesapces(new_cls, _namespaces)
+    return new_cls
