@@ -1,7 +1,8 @@
+from typing import Optional
+
 import rdflib
 
 from ontolutils import QUDT_UNIT
-from typing import Optional
 
 qudt_lookup = {
     's': QUDT_UNIT.SEC,  # time
@@ -28,6 +29,8 @@ qudt_lookup = {
     'm s-2': QUDT_UNIT.M_PER_SEC2,
     # velocity squared
     'm2/s2': QUDT_UNIT.M2_PER_SEC2,
+    'm^2/s^2': QUDT_UNIT.M2_PER_SEC2,
+    'm^2/s': QUDT_UNIT.M2_PER_SEC2,
     'm2/s^2': QUDT_UNIT.M2_PER_SEC2,
     'm2/s**2': QUDT_UNIT.M2_PER_SEC2,
     'm2 s^-2': QUDT_UNIT.M2_PER_SEC2,
@@ -130,6 +133,7 @@ qudt_lookup = {
     'Pa': QUDT_UNIT.PA,
     'pascal': QUDT_UNIT.PA,
     'Pascal': QUDT_UNIT.PA,
+    'kg/m/s**2': QUDT_UNIT.PA,
     # mass
     'kg': QUDT_UNIT.KiloGM,
     'kilogram': QUDT_UNIT.KiloGM,
@@ -149,6 +153,8 @@ qudt_lookup = {
     # torque
     'N m': QUDT_UNIT.N_M,
     'N*m': QUDT_UNIT.N_M,
+    'kg*m**2/s**2': QUDT_UNIT.N_M,
+    # radian
     'rad': QUDT_UNIT.RAD,
     # force
     'N': QUDT_UNIT.N,
@@ -186,6 +192,7 @@ qudt_lookup = {
     'K m': QUDT_UNIT.K_M,
     'mol m-2 s-1': QUDT_UNIT.MOL_PER_M2_SEC,
     'mol m-2 s-1 m-1': QUDT_UNIT.MOL_PER_M2_SEC_M,
+    'mol m-2 s-1 sr-1': QUDT_UNIT.MOL_PER_M2_SEC_SR,
     'W m-2 m-1': QUDT_UNIT.W_PER_M2_M,
     'W m-1': QUDT_UNIT.W_PER_M,
     'W m-2': QUDT_UNIT.W_PER_M2,
@@ -193,6 +200,7 @@ qudt_lookup = {
     'W m-2 sr-1': QUDT_UNIT.W_PER_M2_SR,
     'W m-2 m-1 sr-1': QUDT_UNIT.W_PER_M2_M_SR,
     'm3 s-2': QUDT_UNIT.M3_PER_SEC2,
+    'm^3 s-2': QUDT_UNIT.M3_PER_SEC2,
     'degree m-1': QUDT_UNIT.DEG_PER_M,
     'W m-1 K-1': QUDT_UNIT.W_PER_M_K,
     'K s-1': QUDT_UNIT.K_PER_SEC,
@@ -216,6 +224,23 @@ qudt_lookup = {
     'mm/pixel': "https://matthiasprobst.github.io/pivmeta#MilliM-PER-PIXEL",  # TODO: define this!!!
     'deg': QUDT_UNIT.DEG,
     'degC': QUDT_UNIT.DEG_C,
+    'W s m-2': QUDT_UNIT.W_SEC_PER_M2,
+    'N m-1': QUDT_UNIT.N_PER_M,
+    'mol mol-1': QUDT_UNIT.MOL_PER_MOL,
+    'm4 s-1': QUDT_UNIT.M4_PER_SEC,
+    'degree s-1': QUDT_UNIT.DEG_PER_SEC,
+    'K Pa s-1': QUDT_UNIT.K_PA_PER_SEC,
+    'W/m2': QUDT_UNIT.W_PER_M2,
+    'm2 s rad-1': QUDT_UNIT.M2_SEC_PER_RAD,
+    'K s': QUDT_UNIT.K_SEC,
+    'radian': QUDT_UNIT.RAD,
+    'sr': QUDT_UNIT.SR,
+    'sr-1': QUDT_UNIT.PER_SR,
+    'dbar': QUDT_UNIT.DeciBAR,
+    'mol m-2 s-1 m-1 sr-1': QUDT_UNIT.MOL_PER_M2_SEC_M_SR,
+    'm year-1': QUDT_UNIT.M_PER_YR,
+    'Pa m s-1': QUDT_UNIT.PA_M_PER_SEC,
+    'Pa m s-2': QUDT_UNIT.PA_M_PER_SEC2
 }
 
 
@@ -237,6 +262,4 @@ def parse_unit(unit_str: str) -> Optional[rdflib.URIRef]:
     KeyError
         If the unit string is not in the lookup table
     """
-    if unit_str == '1e-3':
-        return qudt_lookup['dimensionless']
-    return qudt_lookup.get(unit_str, None)
+    return qudt_lookup[unit_str]
