@@ -2,15 +2,20 @@ import datetime
 import json
 import logging
 import unittest
-from typing import Optional
+from typing import Optional, List
 
 import pydantic
 import rdflib
-from pydantic import EmailStr
+from pydantic import EmailStr, model_validator
+from pydantic import ValidationError
 from pydantic import field_validator, Field
 from rdflib.plugins.shared.jsonld.context import Context
+from typing_extensions import Annotated
 
-from ontolutils import Thing, urirefs, namespaces, get_urirefs, get_namespaces
+import ontolutils
+from ontolutils import Thing, urirefs, namespaces, build, Property
+from ontolutils import as_id
+from ontolutils import get_urirefs, get_namespaces, set_config
 from ontolutils import set_logging_level
 from ontolutils.classes import decorator
 from ontolutils.classes.thing import resolve_iri
@@ -63,6 +68,7 @@ class TestNamespaces(unittest.TestCase):
         # extra fields are allowed and either the model field or its uriref can be used
         agent = Agent(lastName='Doe', age=23)
         self.assertEqual(agent.name, 'Doe')
+        self.assertEqual(agent.lastName, 'Doe')
         self.assertEqual(agent.age, 23)
 
         agent = Agent(age=23)
@@ -101,30 +107,30 @@ class TestNamespaces(unittest.TestCase):
         self.assertEqual(ret, None)
 
     def test_split_URIRef(self):
-        self.assertListEqual(split_URIRef(rdflib.URIRef('http://example.com/')),
-                             ['http://example.com/', ''])
-        self.assertListEqual(split_URIRef(rdflib.URIRef('http://example.com/#test')),
-                             ['http://example.com/#', 'test'])
-        self.assertListEqual(split_URIRef(rdflib.URIRef('http://example.com/test')),
-                             ['http://example.com/', 'test'])
-        self.assertListEqual(split_URIRef(rdflib.URIRef('http://example.com/test#test')),
-                             ['http://example.com/test#', 'test'])
-        self.assertListEqual(split_URIRef(rdflib.URIRef('http://example.com/test:123')),
-                             ['http://example.com/', 'test:123'])
+        self.assertListEqual(split_URIRef(rdflib.URIRef('https://example.com/')),
+                             ['https://example.com/', ''])
+        self.assertListEqual(split_URIRef(rdflib.URIRef('https://example.com/#test')),
+                             ['https://example.com/#', 'test'])
+        self.assertListEqual(split_URIRef(rdflib.URIRef('https://example.com/test')),
+                             ['https://example.com/', 'test'])
+        self.assertListEqual(split_URIRef(rdflib.URIRef('https://example.com/test#test')),
+                             ['https://example.com/test#', 'test'])
+        self.assertListEqual(split_URIRef(rdflib.URIRef('https://example.com/test:123')),
+                             ['https://example.com/', 'test:123'])
 
     def test_thing_custom_prop(self):
         """It is helpful to have the properties equal to the urirefs keys,
         however, this should not be required!"""
 
         @namespaces(foaf='http://xmlns.com/foaf/0.1/',
-                    schema='http://www.schema.org/')
+                    schema='https://www.schema.org/')
         @urirefs(Affiliation='prov:Affiliation',
                  name='schema:name')
         class Affiliation(Thing):
             name: str
 
         @namespaces(foaf='http://xmlns.com/foaf/0.1/',
-                    prov='http://www.w3.org/ns/prov#')
+                    prov='https://www.w3.org/ns/prov#')
         @urirefs(Person='prov:Person',
                  first_name='foaf:firstName',
                  lastName='foaf:lastName',
@@ -169,14 +175,55 @@ class TestNamespaces(unittest.TestCase):
         with self.assertRaises(AttributeError):  # height is not defined!
             p183_from_jsonld.height
 
+    def test_from_jsonld_for_nested_objects(self):
+        @namespaces(prov='https://www.w3.org/ns/prov#')
+        @urirefs(A='prov:A',
+                 name='prov:name')
+        class A(Thing):
+            name: str = None
+
+        @namespaces(prov='https://www.w3.org/ns/prov#')
+        @urirefs(B='prov:B',
+                 a='prov:a')
+        class B(Thing):
+            a: A = None
+
+        @namespaces(prov='https://www.w3.org/ns/prov#')
+        @urirefs(C='prov:C',
+                 b='prov:b')
+        class C(Thing):
+            b: B = None
+
+        @namespaces(prov='https://www.w3.org/ns/prov#')
+        @urirefs(D='prov:D',
+                 c='prov:c')
+        class D(Thing):
+            c: C = None
+
+        aj = A(name="myname").model_dump_jsonld()
+        an = A.from_jsonld(data=aj, limit=1)
+        self.assertEqual("myname", an.name)
+
+        bj = B(a=(A(name="myname"))).model_dump_jsonld()
+        bn = B.from_jsonld(data=bj, limit=1)
+        self.assertEqual("myname", bn.a.name)
+
+        cj = C(b=B(a=(A(name="myname")))).model_dump_jsonld()
+        cn = C.from_jsonld(data=cj, limit=1)
+        self.assertEqual("myname", cn.b.a.name)
+
+        dj = D(c=C(b=B(a=(A(name="myname"))))).model_dump_jsonld()
+        dn = D.from_jsonld(data=dj, limit=1)
+        self.assertEqual("myname", dn.c.b.a.name)
+
     def test_sort_classes(self):
         thing1 = Thing(label='Thing 1', id='_:1')
         thing2 = Thing(label='Thing 2', id='_:2')
         self.assertFalse(thing1 > thing2)
         with self.assertRaises(TypeError):
             thing1 < 4
-        thing1 = Thing(label='Thing 1', id='http://example.com/thing1')
-        thing2 = Thing(label='Thing 2', id='http://example.com/thing2')
+        thing1 = Thing(label='Thing 1', id='https://example.com/thing1')
+        thing2 = Thing(label='Thing 2', id='https://example.com/thing2')
         self.assertTrue(thing1 < thing2)
 
     def test__repr_html_(self):
@@ -202,7 +249,7 @@ class TestNamespaces(unittest.TestCase):
         self.assertEqual(thing_dict['@type'], 'owl:Thing')
 
     def test_decorator(self):
-        self.assertTrue(decorator._is_http_url('http://example.com/'))
+        self.assertTrue(decorator._is_http_url('https://example.com/'))
         self.assertFalse(decorator._is_http_url('example.com/'))
         self.assertFalse(decorator._is_http_url('http:invalid.123'))
 
@@ -318,7 +365,6 @@ class TestNamespaces(unittest.TestCase):
         agent = Agent(name='John Doe')
         self.assertEqual(agent.name, 'John Doe')
         agent_jsonld = agent.model_dump_jsonld()
-        print(agent_jsonld.replace('https://schema', 'http://schema'))
         with self.assertWarns(UserWarning):
             agent.from_jsonld(data=agent_jsonld.replace('https://schema', 'http://schema'),
                               limit=1)
@@ -336,12 +382,12 @@ class TestNamespaces(unittest.TestCase):
             """
             mbox: EmailStr = None
 
-        @namespaces(schema="http://schema.org/")
+        @namespaces(schema="https://schema.org/")
         @urirefs(Organization='prov:Organization')
         class Organization(Agent):
             """Pydantic Model for https://www.w3.org/ns/prov/Agent"""
 
-        @namespaces(schema="http://schema.org/")
+        @namespaces(schema="https://schema.org/")
         @urirefs(Person='foaf:Person',
                  affiliation='schema:affiliation')
         class Person(Agent):
@@ -387,37 +433,184 @@ class TestNamespaces(unittest.TestCase):
         self.assertEqual(Agent.iri('mbox'), 'http://xmlns.com/foaf/0.1/mbox')
         self.assertEqual(Agent.iri('mbox', compact=True), 'foaf:mbox')
 
-    def test_person(self):
-        @namespaces(prov="http://www.w3.org/ns/prov#",
-                    foaf="http://xmlns.com/foaf/0.1/")
+    def test_use_as_id(self):
+        @namespaces(prov="https://www.w3.org/ns/prov#",
+                    foaf="http://xmlns.com/foaf/0.1/",
+                    m4i="http://w3id.org/nfdi4ing/metadata4ing#"
+                    )
         @urirefs(Person='prov:Person',
                  firstName='foaf:firstName',
-                 # lastName=FOAF.lastName,
                  lastName='foaf:lastName',
+                 orcidId='m4i:orcidId',
                  mbox='foaf:mbox')
         class Person(Thing):
             firstName: str
             lastName: str = None
             mbox: EmailStr = None
+            orcidId: str = Field(default=None, alias="orcid_id")
 
-        p = Person(id="local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4", firstName='John', lastName='Doe')
+            @model_validator(mode="before")
+            def _change_id(self):
+                return as_id(self, "orcidId")
+
+        p = Person(
+            id="local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4",
+            firstName='John',
+            lastName='Doe',
+            orcidId='https://orcid.org/0000-0001-8729-0482', )
         jsonld = {
             "@context": {
+                'm4i': 'http://w3id.org/nfdi4ing/metadata4ing#',
                 "owl": "http://www.w3.org/2002/07/owl#",
+                "prov": "https://www.w3.org/ns/prov#",
                 "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
                 "foaf": "http://xmlns.com/foaf/0.1/",
-                "prov": "http://www.w3.org/ns/prov#",
             },
-            "@id": "local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4",
             "@type": "prov:Person",
             "foaf:firstName": "John",
-            "foaf:lastName": "Doe"
+            "foaf:lastName": "Doe",
+            "m4i:orcidId": "https://orcid.org/0000-0001-8729-0482",
+            "@id": "local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4",
         }
-        print(p.model_dump_jsonld())
 
         self.assertDictEqual(json.loads(p.model_dump_jsonld()),
                              jsonld)
 
+        p_without_http_orcid = Person(
+            firstName='John',
+            lastName='Doe',
+            orcidId='0000-0001-8729-0482')
+        self.assertEqual(json.loads(p_without_http_orcid.model_dump_jsonld())["@id"],
+                         p_without_http_orcid.id)
+        self.assertEqual(json.loads(p_without_http_orcid.model_dump_jsonld())["m4i:orcidId"],
+                         '0000-0001-8729-0482')
+
+        p = Person(
+            firstName='John',
+            lastName='Doe',
+            orcidId='https://orcid.org/0000-0001-8729-0482', )
+        jsonld = {
+            "@context": {
+                'm4i': 'http://w3id.org/nfdi4ing/metadata4ing#',
+                "owl": "http://www.w3.org/2002/07/owl#",
+                "prov": "https://www.w3.org/ns/prov#",
+                "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
+                "foaf": "http://xmlns.com/foaf/0.1/",
+            },
+            "@type": "prov:Person",
+            "foaf:firstName": "John",
+            "foaf:lastName": "Doe",
+            "m4i:orcidId": "https://orcid.org/0000-0001-8729-0482",
+            "@id": "https://orcid.org/0000-0001-8729-0482",
+        }
+
+        self.assertDictEqual(json.loads(p.model_dump_jsonld()),
+                             jsonld)
+
+    def test_use_as_id_V2(self):
+        @namespaces(schema="https://schema.org/",
+                    foaf="http://xmlns.com/foaf/0.1/",
+                    )
+        @urirefs(Orga='prov:Organization',
+                 name="schema:name",
+                 identifier="schema:identifier",
+                 mbox='foaf:mbox')
+        class Orga(Thing):
+            identifier: str = Field(default=None, alias="identifier")
+            name: str = Field(default=None, alias="name")
+            mbox: EmailStr = None
+
+            @model_validator(mode="before")
+            def _change_id(self):
+                return as_id(self, "identifier")
+
+        @namespaces(prov="https://www.w3.org/ns/prov#",
+                    foaf="http://xmlns.com/foaf/0.1/",
+                    m4i="http://w3id.org/nfdi4ing/metadata4ing#"
+                    )
+        @urirefs(Person='prov:Person',
+                 firstName='foaf:firstName',
+                 lastName='foaf:lastName',
+                 orcidId='m4i:orcidId',
+                 mbox='foaf:mbox')
+        class Person(Thing):
+            firstName: str
+            lastName: str = None
+            mbox: EmailStr = None
+            orcidId: str = Field(default=None, alias="orcid_id")
+            affiliation: Orga = None
+
+            @model_validator(mode="before")
+            def _change_id(self):
+                return as_id(self, "orcidId")
+
+        p = Person(
+            id="local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4",
+            firstName='John',
+            lastName='Doe',
+            orcidId='https://orcid.org/0000-0001-8729-0482',
+            affiliation=Orga(identifier='https://example.org/123', name='Orga 1')
+        )
+        # Person was created with an explicit ID
+        self.assertEqual(p.id, "local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4")
+
+        p = Person(
+            firstName='John',
+            lastName='Doe',
+            orcidId='https://orcid.org/0000-0001-8729-0482',
+            affiliation=Orga(identifier='https://example.org/123', name='Orga 1')
+        )
+
+        pdict = json.loads(p.model_dump_jsonld())
+        self.assertEqual(pdict['@id'], 'https://orcid.org/0000-0001-8729-0482')
+        self.assertEqual(pdict['affiliation']["@id"], 'https://example.org/123')
+
+    def test_use_as_id_V3(self):
+        @namespaces(schema="https://schema.org/",
+                    foaf="http://xmlns.com/foaf/0.1/",
+                    )
+        @urirefs(Orga='prov:Organization',
+                 name="schema:name",
+                 identifier="schema:identifier",
+                 mbox='foaf:mbox')
+        class Orga(Thing):
+            identifier: str = Field(default=None, alias="identifier")
+            name: str = Field(default=None, alias="name")
+            mbox: EmailStr = None
+
+            @model_validator(mode="after")
+            def _change_id(self):
+                return as_id(self, "identifier")
+
+        @namespaces(prov="https://www.w3.org/ns/prov#",
+                    foaf="http://xmlns.com/foaf/0.1/",
+                    m4i="http://w3id.org/nfdi4ing/metadata4ing#"
+                    )
+        @urirefs(Person='prov:Person',
+                 firstName='foaf:firstName',
+                 lastName='foaf:lastName',
+                 orcidId='m4i:orcidId',
+                 mbox='foaf:mbox')
+        class Person(Thing):
+            firstName: str
+            lastName: str = None
+            mbox: EmailStr = None
+            orcidId: str = Field(default=None, alias="orcid_id")
+            affiliation: Orga = None
+
+            @model_validator(mode="before")
+            def _change_id(self):
+                return as_id(self, "orcidId")
+
+        with self.assertRaises(ValueError):
+            Person(
+                id="local:cde4c79c-21f2-4ab7-b01d-28de6e4aade4",
+                firstName='John',
+                lastName='Doe',
+                orcidId='https://orcid.org/0000-0001-8729-0482',
+                affiliation=Orga(identifier='123', name='Orga 1')
+            )
+
     def test_update_namespace_and_uri(self):
         class CustomPerson(Thing):
             pass
@@ -459,3 +652,137 @@ class TestNamespaces(unittest.TestCase):
         jsonld_dict.pop('@id', None)
         self.assertDictEqual(jsonld_dict,
                              ref_jsonld)
+
+    def test_blank_node_prefix(self):
+        @namespaces(foaf='http://xmlns.com/foaf/0.1/',
+                    prov='https://www.w3.org/ns/prov#')
+        @urirefs(Person='prov:Person',
+                 first_name='foaf:firstName')
+        class Person(Thing):
+            first_name: str = Field(default=None, alias='firstName')
+
+        p = Person(firstName="John")
+        self.assertTrue(p.id.startswith("_:"))
+
+        with set_config(blank_node_prefix_name="local:"):
+            p = Person(firstName="John")
+            self.assertTrue(p.id.startswith("local:"))
+
+        p = Person(firstName="John")
+        self.assertTrue(p.id.startswith("_:"))
+
+        ontolutils.set_config(blank_node_prefix_name="test:")
+
+        p = Person(firstName="John")
+        self.assertTrue(p.id.startswith("test:"))
+
+        ontolutils.set_config(blank_node_prefix_name=None)
+        p = Person(firstName="John")
+        self.assertTrue(p.id.startswith("_:"))
+
+    def test_dynamic_thing(self):
+        MyThing = build(
+            namespace="https://schema.org/",
+            namespace_prefix="schema",
+            class_name="MyThing",
+            properties=[Property(
+                name="about",
+                default=None,
+                property_type=str
+            )]
+        )
+        mything = MyThing(about="my thing")
+        self.assertEqual(mything.about, "my thing")
+
+    def test_dynamic_thing_with_validator(self):
+        def validate_str(cls, value):
+            if "thing" not in value:
+                raise ValueError("Value must contain 'thing'")
+            return value
+
+        PositiveInt = Annotated[int, Field(gt=0)]
+
+        MySpecialThing = build(
+            baseclass=Thing,
+            namespace="https://schema.org/",
+            namespace_prefix="schema",
+            class_name="MyThing",
+            properties=[Property(
+                name="value",
+                default=Field(default=None, alias="val"),
+                property_type=PositiveInt
+            )]
+        )
+        with self.assertRaises(ValidationError):
+            MySpecialThing(value=-1)
+        mst = MySpecialThing(value=3)
+        self.assertEqual(mst.value, 3)
+        mst = MySpecialThing(val=4)
+        self.assertEqual(mst.value, 4)
+
+        MySpecialThing2 = Thing.build(
+            namespace="https://schema.org/",
+            namespace_prefix="schema",
+            class_name="MyThing",
+            properties=[Property(
+                name="value",
+                default=Field(default=None, alias="val"),
+                property_type=PositiveInt
+            )]
+        )
+        with self.assertRaises(ValidationError):
+            MySpecialThing2(value=-1)
+        mst = MySpecialThing2(value=3)
+        self.assertEqual(mst.value, 3)
+        mst = MySpecialThing2(val=4)
+        self.assertEqual(mst.value, 4)
+
+        NegativeInt = Annotated[int, Field(lt=0)]
+        MySubSpecialThing = MySpecialThing.build(
+            namespace="https://schema.org/",
+            namespace_prefix="schema",
+            class_name="MySubSpecialThing",
+            properties=[Property(
+                name="negValue",
+                default=Field(default=None, alias="nval"),
+                property_type=NegativeInt
+            )]
+        )
+        with self.assertRaises(ValidationError):
+            MySubSpecialThing(negValue=1)
+        self.assertIsInstance(MySubSpecialThing(negValue=-1), MySubSpecialThing)
+        mst = MySubSpecialThing(negValue=-3)
+        self.assertEqual(mst.negValue, -3)
+        self.assertEqual(mst.value, None)
+        mst = MySubSpecialThing(nval=-4)
+        self.assertEqual(mst.negValue, -4)
+
+    def test_dynamic_forward_references(self):
+        Person = Thing.build(
+            namespace="https://example.org/",
+            namespace_prefix="ex",
+            class_name="Person",
+            properties=[
+                Property(
+                    name="name",
+                    default=None,
+                    property_type=str
+                ),
+                Property(
+                    name="friends",
+                    default=None,
+                    property_type=List["Person"]
+                )
+            ]
+        )
+        person1 = Person(name="John")
+        person2 = Person(name="Jane", friends=[person1])
+        expected_serialization = """@prefix ex: <https://example.org/> .
+
+[] a ex:Person ;
+    ex:friends [ a ex:Person ;
+            ex:name "John" ] ;
+    ex:name "Jane" .
+
+"""
+        self.assertEqual(expected_serialization, person2.serialize("ttl"))
